@@ -721,6 +721,10 @@ typedef CK_ULONG          CK_CERTIFICATE_TYPE;
  * v2.0 */
 typedef CK_ULONG          CK_ATTRIBUTE_TYPE;
 
+/* The CKF_ARRAY_ATTRIBUTE flag identifies an attribute which
+ * consists of an array of values. */
+#define CKF_ARRAY_ATTRIBUTE    0x40000000
+
 /* The following attribute types are defined: */
 #define CKA_CLASS              0x00000000
 #define CKA_TOKEN              0x00000001
@@ -795,6 +799,9 @@ typedef CK_ULONG          CK_ATTRIBUTE_TYPE;
 #define CKA_RESET_ON_INIT      0x00000301
 #define CKA_HAS_RESET          0x00000302
 
+#define CKA_WRAP_TEMPLATE        (CKF_ARRAY_ATTRIBUTE|0x00000211UL)
+#define CKA_UNWRAP_TEMPLATE      (CKF_ARRAY_ATTRIBUTE|0x00000212UL)
+
 #define CKA_VENDOR_DEFINED     0x80000000
 
 /* For use in storing objects that have an encrypted or otherwise
@@ -862,7 +869,13 @@ typedef CK_ULONG          CK_MECHANISM_TYPE;
 #define CKM_X9_42_DH_HYBRID_DERIVE     0x00000032
 #define CKM_X9_42_MQV_DERIVE           0x00000033
 
-#define CKM_SHA256_RSA_PKCS            0x00000043
+#define CKM_SHA256_RSA_PKCS            0x00000040
+#define CKM_SHA384_RSA_PKCS            0x00000041
+#define CKM_SHA512_RSA_PKCS            0x00000042
+
+#define CKM_SHA256_RSA_PKCS_PSS	       0x00000043
+#define CKM_SHA384_RSA_PKCS_PSS	       0x00000044
+#define CKM_SHA512_RSA_PKCS_PSS	       0x00000045
 
 #define CKM_RC2_KEY_GEN                0x00000100
 #define CKM_RC2_ECB                    0x00000101
@@ -902,6 +915,10 @@ typedef CK_ULONG          CK_MECHANISM_TYPE;
 #define CKM_CDMF_MAC_GENERAL           0x00000144
 #define CKM_CDMF_CBC_PAD               0x00000145
 
+#define CKM_DES_OFB64                  0x00000150
+#define CKM_DES_CFB64                  0x00000152
+#define CKM_DES_CFB8                   0x00000153
+
 #define CKM_MD2                        0x00000200
 
 /* CKM_MD2_HMAC and CKM_MD2_HMAC_GENERAL are new for v2.0 */
@@ -1066,10 +1083,17 @@ typedef CK_ULONG          CK_MECHANISM_TYPE;
 #define CKM_AES_MAC                    0x00001083
 #define CKM_AES_MAC_GENERAL            0x00001084
 #define CKM_AES_CBC_PAD                0x00001085
+#define CKM_AES_CTR                    0x00001086
+#define CKM_AES_GCM                    0x00001087
 #define CKM_DSA_PARAMETER_GEN          0x00002000
 #define CKM_DH_PKCS_PARAMETER_GEN      0x00002001
 #define CKM_X9_42_DH_PARAMETER_GEN     0x00002002
 
+#define CKM_AES_OFB                    0x00002104
+#define CKM_AES_CFB64                  0x00002105
+#define CKM_AES_CFB8                   0x00002106
+#define CKM_AES_CFB128                 0x00002107
+
 #define CKM_VENDOR_DEFINED             0x80000000
 
 typedef CK_MECHANISM_TYPE CK_PTR CK_MECHANISM_TYPE_PTR;
@@ -1383,6 +1407,23 @@ typedef struct CK_RC5_PARAMS {
 
 typedef CK_RC5_PARAMS CK_PTR CK_RC5_PARAMS_PTR;
 
+/* CK_AES_CTR_PARAMS is new for PKCS #11 v2.20 amendment 3 */
+typedef struct CK_AES_CTR_PARAMS {
+    CK_ULONG ulCounterBits;
+    CK_BYTE cb[16];
+} CK_AES_CTR_PARAMS;
+
+typedef CK_AES_CTR_PARAMS CK_PTR CK_AES_CTR_PARAMS_PTR;
+
+typedef struct CK_GCM_PARAMS {
+	CK_BYTE_PTR pIv;
+	CK_ULONG ulIvLen;
+	CK_BYTE_PTR pAAD;
+	CK_ULONG ulAADLen;
+	CK_ULONG ulTagBits;
+} CK_GCM_PARAMS;
+
+typedef CK_GCM_PARAMS CK_PTR CK_GCM_PARAMS_PTR;
 
 /* CK_RC5_CBC_PARAMS provides the parameters to the CKM_RC5_CBC
  * mechanism */
@@ -1548,6 +1589,36 @@ typedef CK_ULONG CK_EXTRACT_PARAMS;
 
 typedef CK_EXTRACT_PARAMS CK_PTR CK_EXTRACT_PARAMS_PTR;
 
+/* RSA mechanism OAEP encoding */
+typedef CK_ULONG CK_RSA_PKCS_OAEP_SOURCE_TYPE;
+typedef CK_ULONG CK_RSA_PKCS_MGF_TYPE;
+typedef CK_RSA_PKCS_MGF_TYPE CK_PTR CK_RSA_PKCS_MGF_TYPE_PTR;
+/* PKCS#1 RSA OAEP Encoding Parameter Sources */
+#define CKZ_DATA_SPECIFIED 0x00000001
+
+/* PKCS#1 Mask Generation Functions */
+#define CKG_MGF1_SHA1   0x00000001
+#define CKG_MGF1_SHA256 0x00000002
+#define CKG_MGF1_SHA384 0x00000003
+#define CKG_MGF1_SHA512 0x00000004
+
+typedef struct CK_RSA_PKCS_OAEP_PARAMS {
+	CK_MECHANISM_TYPE hashAlg;
+	CK_RSA_PKCS_MGF_TYPE mgf;
+	CK_RSA_PKCS_OAEP_SOURCE_TYPE source;
+	CK_VOID_PTR pSourceData;
+	CK_ULONG ulSourceDataLen;
+} CK_RSA_PKCS_OAEP_PARAMS;
+
+typedef CK_RSA_PKCS_OAEP_PARAMS CK_PTR CK_RSA_PKCS_OAEP_PARAMS_PTR;
+
+typedef struct CK_RSA_PKCS_PSS_PARAMS {
+	CK_MECHANISM_TYPE hashAlg;
+	CK_RSA_PKCS_MGF_TYPE mgf;
+	CK_ULONG sLen;
+} CK_RSA_PKCS_PSS_PARAMS;
+
+typedef CK_RSA_PKCS_PSS_PARAMS CK_PTR CK_RSA_PKCS_PSS_PARAMS_PTR;
 
 /* CK_FUNCTION_LIST is a structure holding a Cryptoki spec
  * version and pointers of appropriate types to all the
@@ -1857,7 +1928,6 @@ struct CK_FUNCTION_LIST {
 	CK_C_WaitForSlotEvent C_WaitForSlotEvent;
 };
 
-
 #ifdef __cplusplus
 }
 #endif
